Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
Types ¶
type Span ¶
type Span interface {
// End completes the Span. The Span is considered complete and ready to be
// delivered through the rest of the telemetry pipeline after this method
// is called. Therefore, updates to the Span are not allowed after this
// method has been called.
End()
}
Span is the individual component of a trace. It represents a single named and timed operation of a workflow that is traced. A Tracer is used to create a Span and it is then up to the operation the Span represents to properly end the Span when the operation itself ends.
type Tracer ¶
type Tracer interface {
// Start creates a span and a context.Context containing the newly-created
// span.
//
// If the context.Context provided in `ctx` contains a Span then the
// newly-created Span will be a child of that span, otherwise it will be a
// root span.
//
// Any Span that is created MUST also be ended. This is the responsibility
// of the user. Implementations of this API may leak memory or other
// resources if Spans are not ended.
Start(context.Context, string, string) (context.Context, Span)
}
Tracer is the interface that all tracers must implement.
Click to show internal directories.
Click to hide internal directories.